# Notes: https://gitea.lemnoslife.com/Benjamin_Loison/Robust_image_source_identification_on_modern_smartphones/issues/25

import os
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt
import sys

sys.path.insert(0, '../../algorithms/image_utils/')

from image_utils import showImageWithMatplotlib, randomGaussianImage, toPilImage

sys.path.insert(0, '../../algorithms/context_adaptive_interpolator/')

from context_adaptive_interpolator import contextAdaptiveInterpolator

from skimage.restoration import denoise_tv_chambolle

datasetPath = 'no_noise_images'
# Note that contrarily to `datasets/fake/`, here we do not have images being Gaussian with `scale` `1` but actual images with pixel values between 0 and 255.
# In addition to the range difference, note that the distribution in the first set of images was a Gaussian and here is very different and specific.
PRNU_FACTOR = 0.1
IMAGE_SIZE_SHAPE = (469, 704)

np.random.seed(0)

#prnuNpArray = 255 * randomGaussianImage(scale = PRNU_FACTOR, size = IMAGE_SIZE_SHAPE)
prnuPil = Image.open('prnu.png').convert('F')
prnusNpArray = [np.array(prnuPil) * PRNU_FACTOR for PRNU_FACTOR in PRNU_FACTORS]

def isIn256Range(x):
    return 0 <= x and x <= 255

imagesPrnuEstimateNpArray = []

for imageName in os.listdir(datasetPath):
    if imageName.endswith('.png'):
        imagePath = f'{datasetPath}/{imageName}'
        imageWithoutPrnuPil = Image.open(imagePath).convert('F')
        imageWithoutPrnuNpArray = np.array(imageWithoutPrnuPil)

        imageWithPrnuNpArray = imageWithoutPrnuNpArray + prnuNpArray
        #assert all([isIn256Range(extreme) for extreme in [imageWithPrnuNpArray.max(), imageWithPrnuNpArray.min()]]), 'Adding the PRNU resulted in out of 256 bounds image'
        imageWithPrnuPil = toPilImage(imageWithPrnuNpArray)
        #imagePrnuEstimatePil = contextAdaptiveInterpolator(imageWithPrnuPil.load(), imageWithPrnuPil)
        #imagePrnuEstimateNpArray = np.array(imagePrnuEstimatePil)
        imagePrnuEstimateNpArray = imageWithPrnuNpArray - denoise_tv_chambolle(imageWithPrnuNpArray, weight=0.2, channel_axis=-1)

        imagesPrnuEstimateNpArray += [imagePrnuEstimateNpArray]

showImageWithMatplotlib(np.array(imagesPrnuEstimateNpArray).mean(axis = 0))

